#!/usr/bin/env python
"""
This file contains a python commands example for the VLAN module.
Python commands syntax is very similar to the SwitchX SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
"""
import sys, os
from python_sdk_api.sx_api import * 
from test_infra_common import *

tagged_dict = {0: 'Tagged Member', 1: 'Untagged Member'}

old_stdout = redirect_stdout()
rc, handle = sx_api_open(None)
sys.stdout = os.fdopen(old_stdout, 'w')

swid_cnt_p = new_uint32_t_p()
uint32_t_p_assign(swid_cnt_p, 0)
rc = sx_api_port_swid_list_get(handle, None, swid_cnt_p)
if rc != SX_STATUS_SUCCESS:
    print "sx_api_port_swid_list_get failed, rc = %d" % (rc)
    exit()

swid_cnt = uint32_t_p_value(swid_cnt_p)
swid_list_p = new_sx_swid_t_arr(swid_cnt)
rc = sx_api_port_swid_list_get(handle, swid_list_p, swid_cnt_p)
if rc != SX_STATUS_SUCCESS:
    print "sx_api_port_swid_list_get failed, rc = %d" % (rc)
    exit()
swid_cnt = uint32_t_p_value(swid_cnt_p)
printed_header = False

for i in range(0, swid_cnt):
    swid = sx_swid_t_arr_getitem(swid_list_p, i)

    for vid in xrange(1, 4095):
        port_cnt_p = new_uint32_t_p()
        uint32_t_p_assign(port_cnt_p, 0)
        rc = sx_api_vlan_ports_get(handle, swid, vid, None, port_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            print "sx_api_vlan_ports_get failed, rc = %d" % (rc)
            exit()
        port_cnt = uint32_t_p_value(port_cnt_p)
        if port_cnt > 0:
            if not printed_header:
                print "================================================="
                print "|%6s|%6s|%15s|%17s|" % ("SWID", "VLAN", "Log Port ID", "Egress State")
                print "================================================="
                printed_header = True
            
            print "|%6s %6s|" % (swid, vid),

        vlan_port_list_p = new_sx_vlan_ports_t_arr(port_cnt)
        rc = sx_api_vlan_ports_get(handle, swid, vid, vlan_port_list_p, port_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            print "sx_api_vlan_ports_get failed, rc = %d" % (rc)
            exit()
        port_cnt = uint32_t_p_value(port_cnt_p)
        for j in xrange(0, port_cnt):
            vlan_ports = sx_vlan_ports_t_arr_getitem(vlan_port_list_p, j)
            port_id_str = "0x%x" % vlan_ports.log_port
            if j == 0:
                print "%14s|%17s|" % (port_id_str, tagged_dict[vlan_ports.is_untagged])
            else:
                print "|%6s %6s|%15s|%17s|" % ("", "", port_id_str, tagged_dict[vlan_ports.is_untagged])
            if j == (port_cnt - 1):
                print "================================================="
            else:
                print "|%6s %6s|----------------------------------" % ("", "")


